import os
import sys

import click

from showmereqs.analyze import get_third_party_imports
from showmereqs.generate import generate_reqs
from showmereqs.package_info import PackageInfo

logo = r"""\033[92m
 ____  _                    __  __       ____                
/ ___|| |__   _____  __  _ |  \/  | ___ |  _ \ ___  __ _ ___ 
\___ \| '_ \ / _ \ \/  \/ || |\/| |/ _ \| |_) / _ \/ _` / __|
 ___) | | | | (_) \  /\  / | |  | |  __/|  _ <  __/ (_| \__ \
|____/|_| |_|\___/ \/  \/  |_|  |_|\___||_| \_\___|\__, |___/
                                                      \_|     
\033[0m"""


@click.command()
@click.argument(
    "path",
    type=click.Path(exists=True, file_okay=True, dir_okay=True, resolve_path=True),
    default=".",
)
@click.option(
    "--outdir", "-o", help="path to output directory, default is current path"
)
@click.option(
    "--force",
    "-f",
    is_flag=True,
    default=False,
    help="whether to force overwrite output file",
)
@click.option(
    "--no-detail",
    "-nd",
    is_flag=True,
    default=False,
    help="detailed information in requirements.txt",
)
@click.option(
    "--verbose",
    "-v",
    is_flag=True,
    default=False,
    help="whether to show detailed information",
)
def main(**kwargs) -> None:
    """Analyze Python project dependencies and generate requirements.txt"""
    print(logo)
    if kwargs.get("outdir", None) is None:
        kwargs["outdir"] = kwargs["path"]
    check_outdir(kwargs)

    third_party_imports = get_third_party_imports(kwargs["path"])
    third_party_package_infos = [
        PackageInfo(import_name) for import_name in third_party_imports
    ]
    generate_reqs(third_party_package_infos, **kwargs)

    print(f"\033[92mgenerate {kwargs['outdir']}/requirements.txt successfully\033[0m")


def check_outdir(kwargs):
    if os.path.exists(kwargs["outdir"]):
        if not kwargs["force"] and os.path.exists(
            os.path.join(kwargs["outdir"], "requirements.txt")
        ):
            print(
                f"file {kwargs['outdir']}/requirements.txt already exists, use -f to force overwrite"
            )
            sys.exit(1)
    else:
        print(f"create directory {kwargs['outdir']}")
        os.makedirs(kwargs["outdir"], exist_ok=True)


if __name__ == "__main__":
    main()



# tests/test_main.py
import os
import shutil
from pathlib import Path

import pytest
from click.testing import CliRunner

from showmereqs.main import main


@pytest.fixture
def runner():
    """create a cli test runner"""
    return CliRunner()


@pytest.fixture
def temp_project(tmp_path):
    """create a temp project, copy all_kinds_imports to it"""
    template_dir = Path(__file__).parent / "all_kinds_imports"
    shutil.copytree(template_dir, tmp_path / template_dir.name)
    return tmp_path


def exist(path: Path):
    return os.path.exists(path)


def success_in(output: str) -> bool:
    return "requirements.txt successfully" in output


def test_basic_usage(runner: CliRunner, temp_project: Path):
    """test basic usage"""

    result = runner.invoke(main, [str(temp_project)])
    assert result.exit_code == 0
    assert success_in(result.output)
    assert exist(temp_project / "requirements.txt")


def test_force_overwrite(runner: CliRunner, temp_project: Path):
    """test --force -f option"""

    # create a file
    req_txt = temp_project / "requirements.txt"
    req_txt.write_text("old content")

    # not use -f, should fail
    result = runner.invoke(main, [str(temp_project)])
    assert result.exit_code != 0
    assert req_txt.read_text() == "old content"

    # use -f, should success
    result = runner.invoke(main, [str(temp_project), "-f"])
    assert result.exit_code == 0
    assert req_txt.read_text() != "old content"


def test_custom_outdir(runner: CliRunner, temp_project: Path):
    """test --outdir -o option"""

    custom_dir = temp_project / "custom_dir"
    result = runner.invoke(main, [str(temp_project), "-o", str(custom_dir)])
    assert result.exit_code == 0
    assert exist(custom_dir / "requirements.txt")


def test_invalid_path(runner: CliRunner):
    """test invalid path"""
    result = runner.invoke(main, ["non_existent_path"])
    assert result.exit_code != 0


def test_no_detail_option(runner: CliRunner, temp_project: Path):
    """test --no-detail -nd option"""
    req_txt = temp_project / "requirements.txt"
    result = runner.invoke(main, [str(temp_project), "--no-detail"])

    assert result.exit_code == 0
    assert success_in(result.output)
    assert exist(req_txt)
    txt = req_txt.read_text()

    assert "[package]" not in txt
    assert "[import name]" not in txt



import sys
from pathlib import Path

from showmereqs.package_info import PackageInfo


def get_block_txt(
    header: str,
    max_row_len: int,
    infos: list[tuple[str, str]],
    row_prefix: str = "",
    detail: bool = True,
):
    if len(infos) == 0:
        return

    block_txt = f"# {header}\n"
    _txt = "[package]"
    block_txt += f"# {_txt: <{max_row_len+1}}[import name]\n"

    block_txt = block_txt if detail else ""
    for info in infos:
        version_txt = info[0]
        block_txt += f"{row_prefix}{version_txt: <{max_row_len+1}}"
        block_txt += info[1] if detail else ""
        block_txt += "\n"

    block_txt += "\n"
    return block_txt


def sort_row_txts(row_txts: list[tuple[bool, bool, str, str]]):
    full_info = [(x[2], x[3]) for x in row_txts if x[0] and x[1]]
    no_version = [(x[2], x[3]) for x in row_txts if not x[0] and x[1]]
    no_package_name = [(x[2], x[3]) for x in row_txts if x[0] and not x[1]]
    no_info = [(x[2], x[3]) for x in row_txts if not x[0] and not x[1]]

    full_info.sort()
    no_version.sort()
    no_package_name.sort()
    no_info.sort()
    return full_info, no_version, no_package_name, no_info


def generate_reqs(package_infos: list[PackageInfo], **kwargs):
    detail = not kwargs["no_detail"]

    max_package_len = 0
    row_txts: list[tuple[bool, bool, str, str]] = []
    for package_info in package_infos:
        max_package_len = max(
            max_package_len,
            len(package_info.format_version_info()),
        )
        row_txts.append(package_info.format_row())

    # sorted_row_txts is (full_info, no_version, no_package_name, no_info)
    sorted_row_txts = sort_row_txts(row_txts)
    header_txts = [
        "package with version",
        "package without version",
        "package without package name",
        "package without version and package name",
    ]
    prefix_txts = ["", "", "# ", "# "]
    with open(Path(kwargs["outdir"]) / "requirements.txt", "w") as f:
        f.write("# generated by showmereqs\n")
        f.write(f"# python version: {sys.version}\n\n")

        for i, row_txts in enumerate(sorted_row_txts):
            block_txt = get_block_txt(
                header=header_txts[i],
                infos=row_txts,
                max_row_len=max_package_len,
                row_prefix=prefix_txts[i],
                detail=detail,
            )
            if block_txt:
                f.write(block_txt)
